@@ -42,7 +42,7 @@
 get_message(Jabber=#jabber{regexp=RegExp}) when RegExp /= undefined->
     put(regexp, RegExp),
     get_message(Jabber#jabber{regexp=undefined});
-get_message(Jabber=#jabber{type = 'wait'}) ->
+get_message(_Jabber=#jabber{type = 'wait'}) ->
     << >>;
 get_message(Jabber=#jabber{id=user_defined, username=User,passwd=Pwd,type = 'connect'}) ->
     ts_user_server:add_to_connected({User,Pwd}),
@@ -112,7 +112,7 @@ get_message(Jabber=#jabber{type = 'chat', dest=random, domain=Domain,user_server
 get_message(Jabber=#jabber{type = 'chat', dest=unique, domain=Domain,user_server=UserServer})->
     {Dest, _} = ts_user_server:get_first(UserServer),
     message(Dest, Jabber, Domain);
-get_message(Jabber=#jabber{type = 'chat', id=_Id, dest = undefined, domain=Domain}) ->
+get_message(_Jabber=#jabber{type = 'chat', id=_Id, dest = undefined, domain=_Domain}) ->
     %% this can happen if previous is set but undefined, skip
     ts_mon:add({ count, error_no_previous }),
     << >>;
@@ -223,6 +223,13 @@ get_message(#jabber{type = 'muc:exit', room = Room, muc_service = Service, nick
 
 get_message(Jabber=#jabber{id=user_defined}) ->
     get_message2(Jabber);
+
+%% Privacy lists benchmark support
+get_message(#jabber{type = 'privacy:get_names', username = Name, id = Id, domain = Domain}) ->
+    privacy_get_names(username(Name, Id), Domain);
+get_message(#jabber{type = 'privacy:set_active', username = Name, id = Id, domain = Domain}) ->
+    privacy_set_active(username(Name, Id), Domain);
+
 get_message(Jabber=#jabber{username=Name, passwd=Passwd, id=Id}) ->
     FullName = username(Name, Id),
     FullPasswd = password(Passwd,Id),
@@ -387,7 +394,7 @@ auth_sasl_bind(#jabber{username=Name,passwd=Passwd,domain=Domain})->
 %%----------------------------------------------------------------------
 %% Func: auth_sasl_bind/3
 %%----------------------------------------------------------------------
-auth_sasl_bind(Username, _Passwd, Domain) ->
+auth_sasl_bind(_Username, _Passwd, _Domain) ->
  list_to_binary(["<iq type='set' id='",ts_msg_server:get_id(list),
 "' ><bind xmlns='urn:ietf:params:xml:ns:xmpp-bind'><resource>tsung</resource></bind></iq>"]).
 
@@ -402,7 +409,7 @@ auth_sasl_session(#jabber{username=Name,passwd=Passwd,domain=Domain})->
 %%----------------------------------------------------------------------
 %% Func: auth_sasl_session/3
 %%----------------------------------------------------------------------
-auth_sasl_session(Username, _Passwd, _Domain) ->
+auth_sasl_session(_Username, _Passwd, _Domain) ->
  list_to_binary(["<iq type='set' id='",ts_msg_server:get_id(list),
 "' ><session xmlns='urn:ietf:params:xml:ns:xmpp-session' /></iq>"]).
 
@@ -608,6 +615,33 @@ muc_exit(Room,Nick, Service) ->
     Result.
 
 
+%%%----------------------------------------------------------------------
+%%% Func: privacy_get_names/2
+%%% Get names of all privacy lists server stores for the user
+%%%----------------------------------------------------------------------
+privacy_get_names(User, Domain) ->
+    Jid = [User,"@",Domain,"/tsung"],
+    Req = ["<iq from='", Jid, "' type='get' id='getlist'>",
+               "<query xmlns='jabber:iq:privacy'/>",
+           "</iq>"],
+    list_to_binary(Req).
+
+%%%----------------------------------------------------------------------
+%%% Func: privacy_set_active/2
+%%% Set the list named according to pattern "<user>@<domain>_list"
+%%% as active
+%%%----------------------------------------------------------------------
+privacy_set_active(User, Domain) ->
+    Jid = [User,"@",Domain,"/tsung"],
+    List = [User,"@",Domain,"_list"],
+    Req = ["<iq from='", Jid, "' type='set' id='active1'>",
+              "<query xmlns='jabber:iq:privacy'>",
+                  "<active name='", List, "'/>",
+              "</query>",
+           "</iq>"],
+    list_to_binary(Req).
+
+
 %%%----------------------------------------------------------------------
 %%% Func: username/2
 %%% Generate the username given a prefix and id
