@@ -147,8 +147,17 @@ init_seed(A) when is_integer(A)->
     %% have the same seed, therefore, we need to know the id of the
     %% node to set a reproductible but different seed for each launcher.
     Id=get_node_id(),
-    ?LOGF("Seeding with ~p on node ~p~n",[Id,node()],?DEB),
-    random:seed(Id,Id,A);
+    ?DebugF("Seeding with ~p on node ~p~n",[Id,node()]),
+    random:seed(1000*Id,-1000*A*Id,1000*A*A);
+init_seed({A,B}) when is_integer(A) and is_integer(B)->
+    Id=get_node_id(),
+    ?DebugF("Seeding with ~p ~p ~p on node ~p~n",[A,B,Id,node()]),
+    %% init_seed with 2 args is called by ts_client, with increasing
+    %% values of A, and fixed B. If the seeds are too closed, the
+    %% initial pseudo random values will be quite closed to each
+    %% other. Trying to avoid this by using a multiplier big enough
+    %% (because the algorithm use mod 30XXX , see random.erl).
+    random:seed(1000*A*A,-1000*B*B,1000*Id*Id);
 init_seed({A,B,C}) ->
     random:seed(A,B,C).
 
@@ -239,9 +248,12 @@ mkey1search(List, Key) ->
 
 %% close socket if it exists
 close_socket(_Protocol, none) -> ok;
-close_socket(gen_tcp, Socket)-> gen_tcp:close(Socket);
-close_socket(ssl, Socket)    -> ssl:close(Socket);
-close_socket(gen_udp, Socket)-> gen_udp:close(Socket).
+close_socket(gen_tcp, Socket) -> gen_tcp:close(Socket);
+close_socket(gen_tcp6, Socket)-> gen_tcp:close(Socket);
+close_socket(ssl, Socket)     -> ssl:close(Socket);
+close_socket(ssl6, Socket)    -> ssl:close(Socket);
+close_socket(gen_udp, Socket) -> gen_udp:close(Socket);
+close_socket(gen_udp6, Socket)-> gen_udp:close(Socket).
 
 %%----------------------------------------------------------------------
 %% datestr/0
@@ -288,6 +300,12 @@ erl_system_args(extended)->
                    " -kernel inetrc '"++ InetRcFile ++ "'" ;
                _ -> " "
            end,
+    Proto = case init:get_argument(proto_dist) of
+                {ok,[["inet6_tcp"]]}->
+                    ?LOG("IPv6 used for erlang distribution~n",?NOTICE),
+                   " -proto_dist inet6_tcp " ;
+               _ -> " "
+           end,
     ListenMin = case application:get_env(kernel,inet_dist_listen_min) of
                     undefined -> "";
                     {ok, Min} -> " -kernel inet_dist_listen_min " ++ integer_to_list(Min)++ " "
@@ -302,7 +320,7 @@ erl_system_args(extended)->
               "5.3" ++ _Tail     -> " +Mea r10b ";
               _ -> " "
           end,
-    lists:append([BasicArgs, Shared, Hybrid, Smp, Mea, Inet, Threads,ProcessMax,ListenMin,ListenMax]).
+    lists:append([BasicArgs, Shared, Hybrid, Smp, Mea, Inet, Proto, Threads,ProcessMax,ListenMin,ListenMax]).
 
 %%----------------------------------------------------------------------
 %% setsubdir/1
@@ -427,6 +445,7 @@ to_https({request, {body,Data}}) when is_list(Data) ->
     %% body request, no headers
     re:replace(Data,"http://-","https://",[global]);
 to_https({request, S="CONNECT"++Rest}) -> {ok,S};
+to_https({request, []}) -> {ok, []};
 to_https({request, String}) when is_list(String) ->
     EndOfHeader = string:str(String, "\r\n\r\n"),
     Header = string:substr(String, 1, EndOfHeader - 1) ++ "\r\n",
@@ -440,7 +459,9 @@ to_https({request, String}) when is_list(String) ->
     {ok, RealString}.
 
 
-%% @spec from_https(string()) -> {ok, String::string() | Data::iodata}
+%% @spec from_https(string()) -> {ok, string() | iodata()}
+%% @doc replace https links with 'http://-'
+%% @end
 from_https(String) when is_list(String)->
     ReOpts=[{newline,crlf},multiline,global,caseless],
     %% remove Secure from Set-Cookie (TSUN-120)
@@ -522,6 +543,8 @@ spawn_par(Fun, PidFrom, Args) ->
 %%----------------------------------------------------------------------
 inet_setopts(_, none, _) -> %socket was closed before
     none;
+inet_setopts(ssl6, Socket, Opts) ->
+    inet_setopts(ssl, Socket, Opts);
 inet_setopts(ssl, Socket, Opts) ->
     case ssl:setopts(Socket, Opts) of
         ok ->
@@ -532,6 +555,10 @@ inet_setopts(ssl, Socket, Opts) ->
             ?LOGF("Error while setting ssl options ~p ~p ~n", [Opts, Error], ?ERR),
             none
     end;
+inet_setopts(gen_tcp6, Socket,  Opts)->
+    inet_setopts(gen_tcp, Socket,  Opts);
+inet_setopts(gen_udp6, Socket,  Opts)->
+    inet_setopts(gen_udp, Socket,  Opts);
 inet_setopts(_Type, Socket,  Opts)->
     case inet:setopts(Socket, Opts) of
         ok ->
@@ -746,6 +773,10 @@ json_get_bin([Key|Keys],undefined) ->
 json_get_bin([N|Keys],L) when is_integer(N), N =< length(L) ->
     Val =  lists:nth(N,L),
     json_get_bin(Keys,Val);
+json_get_bin([N|Keys], L) when N =:= <<"*">>, is_list(L) ->
+    lists:map(fun(A) -> json_get_bin(Keys,A) end, L);
+json_get_bin([N|Keys],Val) when N =:= <<"*">> ->
+    json_get_bin(Keys,Val);
 json_get_bin([<<"?",Expr/binary>> | Keys],L) when  is_list(L) ->
     case string:tokens(binary_to_list(Expr),"=") of
         [Key,Val] ->
