@@ -57,24 +57,26 @@
 %%% @doc:  read and parse the xml config file
 %%% @end
 %%%----------------------------------------------------------------------
+read(Filename=standard_io, LogDir) ->
+    ?LOG("Reading config file from stdin~n", ?NOTICE),
+    XML = read_stdio(),
+    handle_read(catch xmerl_scan:string(XML,
+                                      [{fetch_path,["/usr/share/tsung/","./"]},
+                                       {validation,true}]),Filename,LogDir);
 read(Filename, LogDir) ->
-    case catch xmerl_scan:file(Filename,
-                               [{fetch_path,["/usr/share/tsung/","./"]},
-                                {validation,true}]) of
-        {ok, Root = #xmlElement{}} ->  % xmerl-0.15
-            ?LOGF("Reading config file: ~s~n", [Filename], ?NOTICE),
-            Table = ets:new(sessiontable, [ordered_set, protected]),
-            {ok, parse(Root, #config{session_tab = Table})};
-        {Root = #xmlElement{}, _Tail} ->  % xmerl-0.19 and up
-            ?LOGF("Reading config file: ~s~n", [Filename], ?NOTICE),
-            Table = ets:new(sessiontable, [ordered_set, protected]),
-            backup_config(LogDir, Filename, Root),
-            {ok, parse(Root, #config{session_tab = Table, proto_opts=#proto_opts{}})};
-        {error,Reason} ->
-            {error, Reason};
-        {'EXIT',Reason} ->
-            {error, Reason}
-    end.
+    ?LOGF("Reading config file: ~s~n", [Filename], ?NOTICE),
+    handle_read(catch xmerl_scan:file(Filename,
+                                      [{fetch_path,["/usr/share/tsung/","./"]},
+                                       {validation,true}]),Filename,LogDir).
+
+handle_read( {Root = #xmlElement{}, _Tail}, Filename, LogDir) ->
+    Table = ets:new(sessiontable, [ordered_set, protected]),
+    backup_config(LogDir, Filename, Root),
+    {ok, parse(Root, #config{session_tab = Table, proto_opts=#proto_opts{}})};
+handle_read({error,Reason},_,_) ->
+    {error, Reason};
+handle_read({'EXIT',Reason},_,_) ->
+    {error, Reason}.
 
 %%%----------------------------------------------------------------------
 %%% Function: parse/2
@@ -100,12 +102,7 @@ parse(Element = #xmlElement{parents = [], attributes=Attrs}, Conf=#config{}) ->
 parse(Element = #xmlElement{name=server, attributes=Attrs}, Conf=#config{servers=ServerList}) ->
     Server = getAttr(Attrs, host),
     Port   = getAttr(integer, Attrs, port),
-    Type = case getAttr(Attrs, type) of
-               "ssl" -> ssl;
-               "tcp" -> gen_tcp;
-               "udp" -> gen_udp;
-               "erlang" -> erlang
-           end,
+    Type = set_net_type(getAttr(Attrs, type)),
 
     lists:foldl(fun parse/2,
         Conf#config{servers = [#server{host=Server,
@@ -114,6 +111,8 @@ parse(Element = #xmlElement{name=server, attributes=Attrs}, Conf=#config{servers
                                      }|ServerList]},
         Element#xmlElement.content);
 
+
+
 %% Parsing the cluster monitoring element (monitor)
 parse(Element = #xmlElement{name=monitor, attributes=Attrs},
       Conf = #config{monitor_hosts=MHList}) ->
@@ -131,11 +130,13 @@ parse(Element = #xmlElement{name=monitor, attributes=Attrs},
                                                          community, ?config(snmp_community)),
                            Version = getAttr(atom,SnmpEl#xmlElement.attributes,
                                                        version, ?config(snmp_version)),
-                           {snmp, {Port, Community, Version}};
+                           %% parse OIDS def
+                           TmpConf = lists:foldl(fun parse/2, Conf#config{oids=[]}, SnmpEl#xmlElement.content),
+                           {snmp, {Port, Community, Version,TmpConf#config.oids}};
                        _ ->
                            {snmp, {?config(snmp_port),
-                            ?config(snmp_community),
-                            ?config(snmp_version)}}
+                                   ?config(snmp_community),
+                                   ?config(snmp_version),[]}}
                    end;
                munin ->
                    case lists:keysearch(munin,#xmlElement.name,
@@ -159,6 +160,21 @@ parse(Element = #xmlElement{name=monitor, attributes=Attrs},
         Conf#config{monitor_hosts = lists:append(MHList, NewMon)},
         Element#xmlElement.content);
 
+
+parse(#xmlElement{name=oid, attributes=Attrs}, Conf=#config{oids=OIDS}) ->
+    OIDStr  = getAttr(Attrs, value),
+    OID  = lists:map(fun erlang:list_to_integer/1, string:tokens(OIDStr,".")),
+    Name = getAttr(atom, Attrs, name),
+    Type = case getAttr(atom, Attrs, type, sample) of
+               sample  -> sample;
+               counter -> sample_counter;
+               sum     -> sum
+           end,
+    Snippet = getAttr(string, Attrs, eval, "fun(X)-> X end."),
+    Fun= ts_utils:eval(Snippet),
+    true = is_function(Fun, 1),
+    Conf#config{oids=[{OID,Name,Type,Fun}| OIDS]};
+
 %%
 parse(Element = #xmlElement{name=load, attributes=Attrs}, Conf) ->
     Loop     = getAttr(integer, Attrs, loop, 0),
@@ -206,7 +222,7 @@ parse(Element = #xmlElement{name=client, attributes=Attrs},
                                                {unix, linux} ->
                                                    {scan, Interface};
                                                OS ->
-                                                   ?LOGF("Scan interface is not supported on OS ~p, abort~n",[OS],?ERR),
+                                                   io:format(standard_error,"Scan interface is not supported on OS ~p, abort~n",[OS]),
                                                    exit({error, scan_interface_not_supported_on_os})
                                            end
                                    end,
@@ -223,8 +239,7 @@ parse(Element = #xmlElement{name=client, attributes=Attrs},
                 %% must be hostname and not ip:
                 case ts_utils:is_ip(Host) of
                     true ->
-                        ?LOGF("ERROR: client config: 'host' attribute must be a hostname, "++
-                              "not an IP ! (was ~p)~n",[Host],?EMERG),
+                        io:format(standard_error,"ERROR: client config: 'host' attribute must be a hostname, "++ "not an IP ! (was ~p)~n",[Host]),
                         exit({error, badhostname});
                     false ->
                         %% add a new client for each CPU
@@ -253,7 +268,12 @@ parse(Element = #xmlElement{name=ip, attributes=Attrs},
                                      StrIP
                              end,
                  ?LOGF("resolving host ~p~n",[ToResolve],?WARN),
-                 {ok,IPtmp} = inet:getaddr(ToResolve,inet),
+                 {ok,IPtmp} = case inet:getaddr(ToResolve,inet) of
+                                  {error,nxdomain} -> % retry with IPv6
+                                      inet:getaddr(ToResolve,inet6);
+                                  Val ->
+                                      Val
+                              end,
                  IPtmp
          end,
     ?LOGF("resolved host ~p~n",[IP],?WARN),
@@ -269,7 +289,7 @@ parse(Element = #xmlElement{name=arrivalphase, attributes=Attrs},
     Phase     = getAttr(integer,Attrs, phase),
     IDuration  = getAttr(integer, Attrs, duration),
     Unit  = getAttr(string,Attrs, unit, "second"),
-    D = 1000 * to_seconds(Unit, IDuration),
+    D = to_milliseconds(Unit, IDuration),
     case lists:keysearch(Phase,#arrivalphase.phase,AList) of
         false ->
             lists:foldl(fun parse/2,
@@ -279,7 +299,7 @@ parse(Element = #xmlElement{name=arrivalphase, attributes=Attrs},
                                                      |AList]},
                         Element#xmlElement.content);
         _ -> % already existing phase, wrong configuration.
-            ?LOGF("Client config error: phase ~p already defined, abort !~n",[Phase],?EMERG),
+            io:format(standard_error,"Client config error: phase ~p already defined, abort !~n",[Phase]),
             exit({error, already_defined_phase})
     end;
 
@@ -290,7 +310,7 @@ parse(Element = #xmlElement{name=user, attributes=Attrs},
     Start   = getAttr(float_or_integer,Attrs, start_time),
     Unit    = getAttr(string,Attrs, unit, "second"),
     Session = getAttr(string,Attrs, session),
-    Delay   = to_seconds(Unit,Start)*1000,
+    Delay   = to_milliseconds(Unit,Start),
     NewUsers= Users++[{Delay,Session}],
     lists:foldl(fun parse/2, Conf#config{static_users = NewUsers},
                 Element#xmlElement.content);
@@ -308,9 +328,9 @@ parse(Element = #xmlElement{name=users, attributes=Attrs},
                     {[],[]} ->
                         exit({invalid_xml,"arrival or interarrival must be specified"});
                     {[], Rate}  when Rate > 0 ->
-                        to_seconds(Unit,Rate) / 1000;
+                        Rate / to_milliseconds(Unit,1);
                     {InterArrival,[]} when InterArrival > 0 ->
-                        1/(1000 * to_seconds(Unit,InterArrival));
+                        1/to_milliseconds(Unit,InterArrival);
                     {_Value, _Value2} ->
                         exit({invalid_xml,"arrivalrate and interarrival can't be defined simultaneously"})
                 end,
@@ -383,7 +403,7 @@ parse(Element = #xmlElement{name=transaction, attributes=Attrs},
 %%%% Parsing the 'if' element
 parse(_Element = #xmlElement{name='if', attributes=Attrs,content=Content},
       Conf = #config{session_tab = Tab, sessions=[CurS|_], curid=Id}) ->
-    VarName = getAttr(atom,Attrs,var),
+    VarName=get_dynvar_name(getAttr(string,Attrs,var)),
     {Rel,Value} = case getAttr(string,Attrs,eq,none) of
                 none -> {neq,getAttr(string,Attrs,neq)};
                 X ->  {eq,X}
@@ -393,7 +413,7 @@ parse(_Element = #xmlElement{name='if', attributes=Attrs,content=Content},
     NewConf = lists:foldl(fun parse/2, Conf#config{curid=Id+1}, Content),
     NewId = NewConf#config.curid,
     ?LOGF("endif in session ~p as id ~p",[CurS#session.id,NewId+1],?INFO),
-    InitialAction = {ctrl_struct, {if_start, Rel, VarName, Value , NewId+1}},
+    InitialAction = {ctrl_struct, {if_start, Rel, VarName, list_to_binary(Value) , NewId+1}},
     %%NewId+1 -> id of the first action after the if
     ets:insert(Tab,{{CurS#session.id,Id+1},InitialAction}),
     NewConf;
@@ -457,7 +477,8 @@ parse(_Element = #xmlElement{name=foreach, attributes=Attrs,content=Content},
 parse(_Element = #xmlElement{name=repeat,attributes=Attrs,content=Content},
     Conf = #config{session_tab = Tab, sessions=[CurS|_], curid=Id}) ->
     MaxRepeat = getAttr(integer,Attrs,max_repeat,20),
-    RepeatName = getAttr(atom,Attrs,name),
+    RepeatName = get_dynvar_name(getAttr(string,Attrs,name)),
+
     [LastElement|_] = lists:reverse([E || E=#xmlElement{} <- Content]),
     case LastElement of
         #xmlElement{name=While,attributes=WhileAttrs}
@@ -470,7 +491,7 @@ parse(_Element = #xmlElement{name=repeat,attributes=Attrs,content=Content},
             Var = getAttr(atom,WhileAttrs,var),
             NewConf = lists:foldl(fun parse/2, Conf#config{curid=Id}, Content),
             NewId = NewConf#config.curid,
-            EndAction = {ctrl_struct,{repeat,RepeatName, While,Rel,Var,Value,Id+1, MaxRepeat}},
+            EndAction = {ctrl_struct,{repeat,RepeatName, While,Rel,Var,list_to_binary(Value),Id+1, MaxRepeat}},
                                  %Id+1 -> id of the first action inside the loop
             ?LOGF("Add repeat action in session ~p as id ~p, Jump to: ~p",
                   [CurS#session.id,NewId+1,Id+1],?INFO),
@@ -535,12 +556,7 @@ parse( #xmlElement{name=change_type, attributes=Attrs},
     Port    = getAttr(integer, Attrs, port),
     Store   = getAttr(atom, Attrs, store, false),
     Restore = getAttr(atom, Attrs, restore, false),
-    PType = case getAttr(Attrs, server_type) of
-               "ssl" -> ssl;
-               "tcp" -> gen_tcp;
-               "udp" -> gen_udp;
-               "erlang" -> erlang
-           end,
+    PType   = set_net_type(getAttr(Attrs, server_type)),
     SessType=case Conf#config.main_sess_type == CType of
                  false -> CurS#session.type;
                  true  -> CType % back to the main type
@@ -793,7 +809,7 @@ parse(Element = #xmlElement{name=setdynvars, attributes=Attrs},
                              Delimiter = getAttr(string,Attrs,delimiter,";"),
                              {setdynvars,file,{Order,FileId,Delimiter},Vars};
                          false ->
-                             ?LOGF("Unknown_file_id ~p in file setdynvars declaration: you forgot to add a file_server option~n",[FileId],?EMERG),
+                             io:format(standard_error, "Unknown_file_id ~p in file setdynvars declaration: you forgot to add a file_server option~n",[FileId]),
                              exit({error, unknown_file_id})
                      end;
                  "random_string" ->
@@ -881,6 +897,11 @@ to_seconds("minute", Val)-> Val*60;
 to_seconds("hour",   Val)-> Val*3600;
 to_seconds("millisecond", Val)-> Val/1000.
 
+to_milliseconds("second", Val)-> Val*1000;
+to_milliseconds("minute", Val)-> Val*60000;
+to_milliseconds("hour",   Val)-> Val*3600000;
+to_milliseconds("millisecond", Val)-> Val.
+
 %%%----------------------------------------------------------------------
 %%% Function: get_default/2
 %%%----------------------------------------------------------------------
@@ -948,6 +969,8 @@ shortnames(Hostname)->
 %%   Use parsed config file to expand all ENTITY
 %% @end
 %%----------------------------------------------------------------------
+backup_config(Dir,standard_io, Config) ->
+    backup_config(Dir, "tsung_stdin.xml", Config);
 backup_config(Dir, Name, Config) ->
     BaseName = filename:basename(Name),
     {ok,IOF}=file:open(filename:join(Dir,BaseName),[write]),
@@ -959,3 +982,28 @@ backup_config(Dir, Name, Config) ->
             ok
     end,
     file:close(IOF).
+
+%% @spec read_stdio()-> string()
+%% @doc Read config from standard input
+%% @end
+read_stdio()->
+    read_stdio(io:get_line(""),[]).
+
+read_stdio(eof, Data)->
+    lists:flatten(Data);
+read_stdio(Data,Acc) ->
+    read_stdio(io:get_line(""),[Acc,Data]).
+
+set_net_type("tcp")  -> gen_tcp;
+set_net_type("tcp6") -> gen_tcp6;
+set_net_type("udp")  -> gen_udp;
+set_net_type("udp6") -> gen_udp6;
+set_net_type("ssl")  -> ssl;
+set_net_type("ssl6") -> ssl6.
+
+get_dynvar_name(VarNameStr) ->
+    %% check if the var name is for an array (myvar[N])
+    case re:run(VarNameStr,"(.+)\[(\d+)\]",[{capture,all_but_first,list},dotall]) of
+        {match,[Name,Index]} -> {list_to_atom(Name),Index};
+        _                    -> list_to_atom(VarNameStr)
+    end.
